/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton.org

Copyright (c) 2002, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/stat.h>

// The purpose of this program is to change a text
// file to have unix style newlines,
// as opposed to dos or mac style ones


void msg_error(char *str, char *filename, int error);

static int ProcessFile(char *filename, void *ptr);
static void translate_and_save(char *buffer,long len, char *filename);

static int verbose = 0;

int main(int argc, char **argv)
{
	int count;
	char *filename;
	

	if( argc < 2 )
	{
		fprintf(stderr,"Usage: %s <files>\n",argv[0]);
		exit(1);
	}

	count = 1;

	if( !strcmp(argv[1], "-v") )
	{
		verbose = 1;
		count++;
		if( argc < 3 )
		{
			fprintf(stderr,"Usage: %s <files>\n",argv[0]);
			exit(1);
		}
	}

start:
	{
		filename = argv[count];
		(void)ProcessFile( filename,NULL );
		count++;
	}
	if( count < argc )
		goto start;

	return 0;
}

static int ProcessFile(char *filename, void *ptr)
{
	FILE *file = NULL;
	char *buffer = NULL;
	long len;
	struct stat stat_rec;

	if( stat(filename, &stat_rec) )
	{
		msg_error("Error from stat",filename,errno);
		goto error;
	}
	if( S_ISDIR( stat_rec.st_mode ) )
	{
		fprintf(stderr,"%s is a directory, skipping\n",filename);
		goto error;
	}

	if( verbose )
		printf("%s\n",filename);

	file = fopen( filename, "rb" );
	if( !file )
	{
		msg_error( "Error opening file for reading",filename,errno);
		goto error;
	}

	if( fseek(file,0,SEEK_END) )
	{
		msg_error( "Error seeking",filename,errno);
		goto error;
	}

	len = ftell(file);
	if( len == EOF )
	{
		msg_error( "Error getting file position",filename,errno);
		goto error;
	}
	
	if( fseek(file,0,SEEK_SET) )
	{
		msg_error("Error seeking",filename,errno);
		goto error;
	}

	buffer = (char *)malloc( sizeof(char) * len );
	if( !buffer )
	{
		msg_error("Couldn't get buffer",filename,errno);
		goto error;
	}
	
	if( fread( (void *)buffer, len, 1, file ) != 1 )
	{
		msg_error("Error reading",filename,errno);
		goto error;
	}

	// close the file for reading so we can open it for
	// writing.
	if( fclose(file) )
		msg_error( "Error closing file",filename,errno);
	file = NULL;
		
	// NOTE: I think it's safe to write over the file
	// because the file will never get longer,
	// it will either get shorter or stay the same
	// length.
	translate_and_save(buffer,len,filename);

error:
	// free the buffer.
	if( buffer )
		free( buffer );
	buffer = NULL;
	// close the file if needed
	if( file )
		(void)fclose(file);
	file = NULL;


	return 0;
}



void msg_error(char *str, char *filename, int error)
{
	fprintf(stderr,"%s: %s, %s\n",
		str,
		filename,
		strerror(error)
		);
}



static void translate_and_save(char *buffer,long len, char *filename)
{
	char *ptr = buffer;
	long count;
	FILE *file = NULL;

	// NOTE: doing it this way fucking changes the creation
	// date of the file which really screws up shit for source code.
//	file = fopen(filename,"wb");
	if( truncate(filename,0) )
	{
		msg_error("Error truncating file",filename,errno);
		return;
	}
	file = fopen(filename,"a");

	if( !file )
	{
		msg_error("Error opening for writing",filename,errno);
		return;
	}

	// dos is 13 10,
	// mac is 13
	// unix is 10
#define kLinefeed 13
#define kNewline 10

	count = 0;
	while(count < len)
	{
		if( *ptr == kLinefeed )
		{
			// if the next is newline,
			// just skip this one,
			// if the next isn't,
			// write this as newline
			if( count == len-1 || ptr[1] != kNewline )
			{
				if( putc( kNewline,file ) == EOF )
				{
					msg_error("Error writing",filename,errno);
					return;
				}
			}
		}
		else
		{
			if( putc( *ptr,file ) == EOF )
			{
				msg_error("Error writing",filename,errno);
				return;
			}
		}
		ptr++;
		count++;
	}

	if( fclose(file ) )
		msg_error("Error closing file afte write",filename,errno);
}

